@@ -1,5 +1,5 @@
 /***************************************************************************
-    begin                : Sun May 15 2005 
+    begin                : Sun May 15 2005
     copyright            : (C) 2005 by Michael Pyne
     email                : michael.pyne@kdemail.net
 ***************************************************************************/
@@ -13,12 +13,14 @@
  *                                                                         *
  ***************************************************************************/
 
-#ifndef JUK_COVERDIALOG_H
-#define JUK_COVERDIALOG_H
+#ifndef COVERDIALOG_H
+#define COVERDIALOG_H
 
-#include "coverdialogbase.h"
+#include "ui_coverdialogbase.h"
 
-class CoverDialog : public CoverDialogBase
+#include <QWidget>
+
+class CoverDialog : public QWidget, public Ui::CoverDialogBase
 {
     Q_OBJECT
 public:
@@ -28,14 +30,14 @@ public:
     virtual void show();
 
 public slots:
-    void slotArtistClicked(QListViewItem *item);
-    void slotContextRequested(QIconViewItem *item, const QPoint &pt);
+    void slotArtistClicked(Q3ListViewItem *item);
+    void slotContextRequested(Q3IconViewItem *item, const QPoint &pt);
 
 private slots:
     void loadCovers();
     void removeSelectedCover();
 };
 
-#endif /* JUK_COVERDIALOG_H */
+#endif /* COVERDIALOG_H */
 
-// vim: set et ts=4 sw=4:
+// vim: set et sw=4 tw=0 sta:
