@@ -4,7 +4,7 @@
     email                : wheeler@kde.org
 
     copyright            : (C) 2008, 2009 by Michael Pyne
-    email                : michael.pyne@kdemail.net
+    email                : mpyne@kde.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -18,8 +18,6 @@
 
 #include "juk.h"
 
-#include <config.h>
-
 #include <kcmdlineargs.h>
 #include <kstatusbar.h>
 #include <kdebug.h>
@@ -61,14 +59,23 @@ using namespace ActionCollection;
 
 JuK* JuK::m_instance;
 
+template<class T>
+void deleteAndClear(T *&ptr)
+{
+    delete ptr;
+    ptr = 0;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
 JuK::JuK(QWidget *parent) :
     KXmlGuiWindow(parent, Qt::WDestructiveClose),
+    m_splitter(0),
+    m_statusLabel(0),
     m_systemTray(0),
-    m_player(PlayerManager::instance()),
+    m_player(new PlayerManager),
     m_shuttingDown(false)
 {
     // Expect segfaults if you change this order.
@@ -96,6 +103,8 @@ JuK::JuK(QWidget *parent) :
     readConfig();
     setupGlobalAccels();
 
+    connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), SLOT(slotAboutToQuit()));
+
     // slotCheckCache loads the cached entries first to populate the collection list
 
     QTimer::singleShot(0, CollectionList::instance(), SLOT(slotCheckCache()));
@@ -118,6 +127,11 @@ JuK* JuK::JuKInstance()
     return m_instance;
 }
 
+PlayerManager *JuK::playerManager() const
+{
+    return m_player;
+}
+
 void JuK::coverDownloaded(const QPixmap &cover)
 {
     QString event(cover.isNull() ? "coverFailed" : "coverDownloaded");
@@ -141,14 +155,14 @@ void JuK::setupLayout()
 {
     new TagTransactionManager(this);
 
-    m_splitter = new PlaylistSplitter(this, "playlistSplitter");
+    m_splitter = new PlaylistSplitter(m_player, this);
     setCentralWidget(m_splitter);
 
     m_statusLabel = new StatusLabel(m_splitter->playlist(), statusBar());
     connect(CollectionList::instance(), SIGNAL(signalCollectionChanged()),
             m_statusLabel, SLOT(updateData()));
     statusBar()->addWidget(m_statusLabel, 1);
-    PlayerManager::instance()->setStatusLabel(m_statusLabel);
+    m_player->setStatusLabel(m_statusLabel);
 
     m_splitter->setFocus();
     resize(750, 500);
@@ -201,6 +215,10 @@ void JuK::setupActions()
     act->setText(i18n("Remove From Playlist"));
     act->setIcon(KIcon("list-remove"));
 
+    act = collection->add<KToggleAction>("crossfadeTracks");
+    act->setText(i18n("Crossfade Between Tracks"));
+    connect(act, SIGNAL(triggered(bool)), m_player, SLOT(setCrossfadeEnabled(bool)));
+
     act = collection->addAction("play", m_player, SLOT(play()));
     act->setText(i18n("&Play"));
     act->setIcon(KIcon("media-playback-start"));
@@ -298,15 +316,11 @@ void JuK::setupActions()
 void JuK::slotSetupSystemTray()
 {
     if(m_toggleSystemTrayAction && m_toggleSystemTrayAction->isChecked()) {
-        m_systemTray = new SystemTray(this);
+        m_systemTray = new SystemTray(m_player, this);
         m_systemTray->setObjectName("systemTray");
-        m_systemTray->show();
 
         m_toggleDockOnCloseAction->setEnabled(true);
         m_togglePopupsAction->setEnabled(true);
-
-        connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), SLOT(slotAboutToQuit()));
-        connect(m_systemTray, SIGNAL(quitSelected()), this, SLOT(slotQuit()));
     }
     else {
         m_systemTray = 0;
@@ -371,6 +385,10 @@ void JuK::readConfig()
         const int maxVolume = 100;
         const int volume = playerConfig.readEntry("Volume", maxVolume);
         m_player->setVolume(volume * 0.01);
+
+        bool enableCrossfade = playerConfig.readEntry("CrossfadeTracks", true);
+        m_player->setCrossfadeEnabled(enableCrossfade);
+        ActionCollection::action<KAction>("crossfadeTracks")->setChecked(enableCrossfade);
     }
 
     // Default to no random play
@@ -418,6 +436,9 @@ void JuK::saveConfig()
     KAction *a = ActionCollection::action<KAction>("loopPlaylist");
     playerConfig.writeEntry("LoopPlaylist", a->isChecked());
 
+    a = ActionCollection::action<KAction>("crossfadeTracks");
+    playerConfig.writeEntry("CrossfadeTracks", a->isChecked());
+
     a = ActionCollection::action<KAction>("albumRandomPlay");
     if(a->isChecked())
         playerConfig.writeEntry("RandomPlay", "AlbumRandomPlay");
@@ -481,11 +502,10 @@ void JuK::slotAboutToQuit()
 {
     m_shuttingDown = true;
 
-    delete m_systemTray;
-    m_systemTray = 0;
-
-    delete m_splitter;
-    m_splitter = 0;
+    deleteAndClear(m_systemTray);
+    deleteAndClear(m_splitter);
+    deleteAndClear(m_player);
+    deleteAndClear(m_statusLabel);
 
     // Playlists depend on CoverManager, so CoverManager should shutdown as
     // late as possible
