@@ -18,19 +18,21 @@
 
 #include "playlistinterface.h"
 
-#include <qhbox.h>
+#include <khbox.h>
+//Added by qt3to4:
+#include <QLabel>
+#include <QEvent>
 
 class QLabel;
 class KSqueezedTextLabel;
 
-class FileHandle;
 
-class StatusLabel : public QHBox, public PlaylistObserver
+class StatusLabel : public KHBox, public PlaylistObserver
 {
     Q_OBJECT
 
 public:
-    StatusLabel(PlaylistInterface *playlist, QWidget *parent = 0, const char *name = 0);
+    explicit StatusLabel(PlaylistInterface *playlist, QWidget *parent = 0);
     virtual ~StatusLabel();
     virtual void updateCurrent();
 
@@ -38,7 +40,7 @@ public slots:
     /**
      * This just sets internal variables that are used by setItemCurrentTime().
      * Please call that method to display the time.
-     */      
+     */
     void setItemTotalTime(int time) { m_itemTotalTime = time; }
     void setItemCurrentTime(int time) { m_itemCurrentTime = time; updateTime(); }
     virtual void updateData();
@@ -62,3 +64,5 @@ private:
 };
 
 #endif
+
+// vim: set et sw=4 tw=0 sta:
