@@ -68,12 +68,12 @@ bool Tag::save()
     TagLib::ID3v2::FrameFactory::instance()->setDefaultTextEncoding(TagLib::String::UTF8);
     TagLib::File *file = MediaFiles::fileFactoryByType(m_fileName);
 
-    if(file && file->isValid() && file->tag() && !file->readOnly()) {
-        file->tag()->setTitle(TagLib::String(m_title.toUtf8().data(), TagLib::String::UTF8));
-        file->tag()->setArtist(TagLib::String(m_artist.toUtf8().data(), TagLib::String::UTF8));
-        file->tag()->setAlbum(TagLib::String(m_album.toUtf8().data(), TagLib::String::UTF8));
-        file->tag()->setGenre(TagLib::String(m_genre.toUtf8().data(), TagLib::String::UTF8));
-        file->tag()->setComment(TagLib::String(m_comment.toUtf8().data(), TagLib::String::UTF8));
+    if(file && !file->readOnly() && file->isValid() && file->tag()) {
+        file->tag()->setTitle(TagLib::String(m_title.toUtf8().constData(), TagLib::String::UTF8));
+        file->tag()->setArtist(TagLib::String(m_artist.toUtf8().constData(), TagLib::String::UTF8));
+        file->tag()->setAlbum(TagLib::String(m_album.toUtf8().constData(), TagLib::String::UTF8));
+        file->tag()->setGenre(TagLib::String(m_genre.toUtf8().constData(), TagLib::String::UTF8));
+        file->tag()->setComment(TagLib::String(m_comment.toUtf8().constData(), TagLib::String::UTF8));
         file->tag()->setTrack(m_track);
         file->tag()->setYear(m_year);
         result = file->save();
@@ -155,16 +155,7 @@ CacheDataStream &Tag::read(CacheDataStream &s)
     }
     }
 
-    // Try to reduce memory usage: share tags that frequently repeat, squeeze others
-
-    m_title.squeeze();
-    m_lengthString.squeeze();
-
-    m_comment = StringShare::tryShare(m_comment);
-    m_artist  = StringShare::tryShare(m_artist);
-    m_album   = StringShare::tryShare(m_album);
-    m_genre   = StringShare::tryShare(m_genre);
-
+    minimizeMemoryUsage();
     return s;
 }
 
@@ -213,9 +204,23 @@ void Tag::setup(TagLib::File *file)
         m_title = i > 0 ? m_fileName.mid(i + 1, j - i - 1) : m_fileName;
     }
 
+    minimizeMemoryUsage();
     m_isValid = true;
 }
 
+void Tag::minimizeMemoryUsage()
+{
+    // Try to reduce memory usage: share tags that frequently repeat, squeeze others
+
+    m_title.squeeze();
+    m_lengthString.squeeze();
+
+    m_comment = StringShare::tryShare(m_comment);
+    m_artist  = StringShare::tryShare(m_artist);
+    m_album   = StringShare::tryShare(m_album);
+    m_genre   = StringShare::tryShare(m_genre);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // related functions
 ////////////////////////////////////////////////////////////////////////////////
