@@ -1,5 +1,5 @@
 /***************************************************************************
-    begin                : Thu Aug 19 2004 
+    begin                : Thu Aug 19 2004
     copyright            : (C) 2002 - 2004 by Michael Pyne
     email                : michael.pyne@kde.org
 ***************************************************************************/
@@ -13,8 +13,8 @@
  *                                                                         *
  ***************************************************************************/
 
-#ifndef _TRACKSEQUENCEITERATOR_H
-#define _TRACKSEQUENCEITERATOR_H
+#ifndef TRACKSEQUENCEITERATOR_H
+#define TRACKSEQUENCEITERATOR_H
 
 #include "playlistitem.h"
 #include "playlistsearch.h"
@@ -45,7 +45,7 @@ public:
      * @param other the TrackSequenceIterator we are copying
      */
     TrackSequenceIterator(const TrackSequenceIterator & other);
-    
+
     /**
      * Default destructor.
      */
@@ -227,6 +227,6 @@ private:
     PlaylistSearch m_albumSearch;
 };
 
-#endif /* _TRACKSEQUENCEITERATOR_H */
+#endif /* TRACKSEQUENCEITERATOR_H */
 
-// vim: set et sw=4:
+// vim: set et sw=4 tw=0 sta:
