@@ -49,6 +49,7 @@
 #include <ktoggleaction.h>
 
 // KMix
+#include "guiprofile.h"
 #include "mixertoolbox.h"
 #include "kmix.h"
 #include "kmixdevicemanager.h"
@@ -58,22 +59,24 @@
 #include "kmixtoolbox.h"
 #include "version.h"
 #include "viewdockareapopup.h"
-//#include "osd.h" // Postponed to KDE4.1
+#include "dialogselectmaster.h"
+
+#include "osdwidget.h"
 
 
 /* KMixWindow
  * Constructs a mixer window (KMix main window)
  */
 KMixWindow::KMixWindow(bool invisible)
-   : KXmlGuiWindow(0),
+   : KXmlGuiWindow(0, Qt::WindowFlags( KDE_DEFAULT_WINDOWFLAGS | Qt::WindowContextHelpButtonHint) ),
    m_showTicks( true ),
-   m_showMenubar(true),
 //   m_isVisible (false),    // initialize, as we don't trigger a hideEvent()
 //   m_visibilityUpdateAllowed( true ),
    m_multiDriverMode (false), // -<- I never-ever want the multi-drivermode to be activated by accident
    m_dockWidget(),
    m_dontSetDefaultCardOnStart (false),
-   _dockAreaPopup(0)
+   _dockAreaPopup(0),
+   _volWA(0)
 {
     setObjectName("KMixWindow");
     // disable delete-on-close because KMix might just sit in the background waiting for cards to be plugged in
@@ -81,11 +84,13 @@ KMixWindow::KMixWindow(bool invisible)
 
    initActions(); // init actions first, so we can use them in the loadConfig() already
    loadConfig(); // Load config before initMixer(), e.g. due to "MultiDriver" keyword
+   initActionsLate(); // init actions that require a loaded config
+   KGlobal::locale()->insertCatalog("kmix-controls");
    initWidgets();
    initPrefDlg();
    MixerToolBox::instance()->initMixer(m_multiDriverMode, m_hwInfoString);
    KMixDeviceManager *theKMixDeviceManager = KMixDeviceManager::instance();
-   recreateGUI();
+   recreateGUI(false);
    fixConfigAfterRead();
    theKMixDeviceManager->initHotplug();
    connect(theKMixDeviceManager, SIGNAL( plugged( const char*, const QString&, QString&)), SLOT (plugged( const char*, const QString&, QString&) ) );
@@ -125,38 +130,35 @@ void KMixWindow::initActions()
    action = actionCollection()->addAction("toggle_channels_currentview");
    action->setText(i18n("Configure &Channels..."));
    connect(action, SIGNAL(triggered(bool) ), SLOT(slotConfigureCurrentView()));
+   action = actionCollection()->addAction( "select_master" );
+   action->setText( i18n("Select Master Channel...") );
+   connect(action, SIGNAL(triggered(bool) ), SLOT(slotSelectMaster()));
 
-   KAction* globalAction = actionCollection()->addAction("increase_volume");
-   globalAction->setText(i18n("Increase Volume"));
-   globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeUp));
-   connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotIncreaseVolume()));
-
-   globalAction = actionCollection()->addAction("decrease_volume");
-   globalAction->setText(i18n("Decrease Volume"));
-   globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeDown));
-   connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotDecreaseVolume()));
-
-   globalAction = actionCollection()->addAction("mute");
-   globalAction->setText(i18n("Mute"));
-   globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeMute));
-   connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotMute()));
-
-   volumeDisplay = new QProgressBar();
-   volumeDisplay->setWindowFlags(Qt::X11BypassWindowManagerHint);
-   QDesktopWidget* desktop = KApplication::kApplication()->desktop();
-   //Getting QRect of the screen where cursor is positioned
-   QRect rect = desktop->screenGeometry(QCursor::pos());
-   int width = (rect.width()/2) - (volumeDisplay->width()/2);
-   int height = (rect.height()/2) - (volumeDisplay->height()/2);
-   width += rect.x();
-   height += rect.y();
-   volumeDisplay->move(width, height);
-   volumeDisplayTimer = new QTimer(this);
-   connect(volumeDisplayTimer, SIGNAL(timeout()), this, SLOT(slotHideVolumeDisplay()));
+   osdWidget = new OSDWidget();
 
    createGUI( "kmixui.rc" );
 }
 
+void KMixWindow::initActionsLate()
+{
+  if ( m_autouseMultimediaKeys ) {
+    KAction* globalAction = actionCollection()->addAction("increase_volume");
+    globalAction->setText(i18n("Increase Volume"));
+    globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeUp));
+    connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotIncreaseVolume()));
+
+    globalAction = actionCollection()->addAction("decrease_volume");
+    globalAction->setText(i18n("Decrease Volume"));
+    globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeDown));
+    connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotDecreaseVolume()));
+
+    globalAction = actionCollection()->addAction("mute");
+    globalAction->setText(i18n("Mute"));
+    globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeMute));
+    connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotMute()));
+  }
+}
+
 
 void KMixWindow::initPrefDlg()
 {
@@ -205,9 +207,12 @@ bool KMixWindow::updateDocking()
       m_dockWidget->deleteLater();
       m_dockWidget = 0L;
    }
-   if ( _dockAreaPopup ) {
+   if ( _volWA ) {
       // If this is called during a master control change, we rather play safe by using deleteLater().
-      _dockAreaPopup->deleteLater();
+      // Note: deleting _volWA also deletes its associated _dockAreaPopup and prevents the action to be left
+      // with a dangling pointer.
+      _volWA->deleteLater();
+      _volWA = 0L;
       _dockAreaPopup = 0L;
    }
 
@@ -223,9 +228,9 @@ bool KMixWindow::updateDocking()
       _dockAreaPopup = new ViewDockAreaPopup(volMenu, "dockArea", Mixer::getGlobalMasterMixer(), 0, (GUIProfile*)0, this);
       _dockAreaPopup->createDeviceWidgets();
 
-      QWidgetAction *volWA = new QWidgetAction(volMenu);
-      volWA->setDefaultWidget(_dockAreaPopup);
-      volMenu->addAction(volWA);
+      _volWA = new QWidgetAction(volMenu);
+      _volWA->setDefaultWidget(_dockAreaPopup);
+      volMenu->addAction(_volWA);
       referenceWidgetForSystray = volMenu;
    }
    m_dockWidget = new KMixDockWidget( this, referenceWidgetForSystray, _dockAreaPopup );
@@ -236,17 +241,22 @@ bool KMixWindow::updateDocking()
 
 void KMixWindow::saveConfig()
 {
+   kDebug() << "About to save config";
    saveBaseConfig();
    saveViewConfig();
    saveVolumes();
 #ifdef __GNUC_
 #warn We must Sync here, or we will lose configuration data. The reson for that is unknown.
 #endif
+
+   kDebug() << "Saved config ... now syncing explicitely";
    KGlobal::config()->sync();
+   kDebug() << "Saved config ... sync finished";
 }
 
 void KMixWindow::saveBaseConfig()
 {
+   kDebug() << "About to save config (Base)";
    KConfigGroup config(KGlobal::config(), "Global");
 
    config.writeEntry( "Size", size() );
@@ -263,6 +273,7 @@ void KMixWindow::saveBaseConfig()
    config.writeEntry( "startkdeRestore", m_onLogin );
    config.writeEntry( "DefaultCardOnStart", m_defaultCardOnStart );
    config.writeEntry( "ConfigVersion", KMIX_CONFIG_VERSION );
+   config.writeEntry( "AutoUseMultimediaKeys", m_autouseMultimediaKeys );
    Mixer* mixerMasterCard = Mixer::getGlobalMasterMixer();
    if ( mixerMasterCard != 0 ) {
       config.writeEntry( "MasterMixer", mixerMasterCard->id() );
@@ -279,10 +290,13 @@ void KMixWindow::saveBaseConfig()
       config.writeEntry( "Orientation","Horizontal" );
    else
       config.writeEntry( "Orientation","Vertical" );
+
+   kDebug() << "Config (Base) saving done";
 }
 
 void KMixWindow::saveViewConfig()
 {
+    kDebug() << "About to save config (View)";
     // Save Views
     for ( int i=0; i<m_wsMixers->count() ; ++i )
     {
@@ -294,6 +308,7 @@ void KMixWindow::saveViewConfig()
             mw->saveConfig( KGlobal::config().data() );
         }
     }
+   kDebug() << "Config (View) saving done";
 }
 
 
@@ -303,6 +318,7 @@ void KMixWindow::saveViewConfig()
  */
 void KMixWindow::saveVolumes()
 {
+   kDebug() << "About to save config (Volume)";
    KConfig *cfg = new KConfig( "kmixctrlrc" );
    for ( int i=0; i<Mixer::mixers().count(); ++i)
    {
@@ -312,6 +328,7 @@ void KMixWindow::saveVolumes()
       }
    }
    delete cfg;
+   kDebug() << "Config (Volume) saving done";
 }
 
 
@@ -339,6 +356,7 @@ void KMixWindow::loadBaseConfig()
    m_configVersion = config.readEntry( "ConfigVersion", 0 );
    // WARNING Don't overwrite m_configVersion with the "correct" value, before having it
    // evaluated. Better only write that in saveBaseConfig()
+   m_autouseMultimediaKeys = config.readEntry( "AutoUseMultimediaKeys", true );
    QString mixerMasterCard = config.readEntry( "MasterMixer", "" );
    QString masterDev = config.readEntry( "MasterMixerDevice", "" );
    //if ( ! mixerMasterCard.isEmpty() && ! masterDev.isEmpty() ) {
@@ -393,16 +411,41 @@ KMixWindow::loadVolumes()
 
 
 
+void KMixWindow::recreateGUIwithoutSavingView()
+{
+	recreateGUI(false);
+}
+
+
 /**
  * Create or recreate the Mixer GUI elements
  */
-void KMixWindow::recreateGUI()
+void KMixWindow::recreateGUI(bool saveConfig)
 {
+   // Find out which of the tabs is currently selected for restoration
+   int current_tab = -1;
+   if (m_wsMixers)
+      current_tab = m_wsMixers->currentIndex();
+
+   // NOTE (coling) This is a bug but I don't have time to find the source.
+   // When returning from "Configure Mixers..." we MUST save, but the
+   // flag comes through as false, presumably due to the rebuildGUI() signal
+   // being tied to the recreateGUIwithoutSavingView() slot.
+   // This should really be fixed :s
+   Q_UNUSED(saveConfig);
    saveViewConfig();  // save the state before recreating
+
+   // Before clearing the mixer widgets, we must increase the refcount on the guiprof to save it deleting the ViewBase object.
+   if ( Mixer::mixers().count() > 0 )
+      for (int i=0; i<Mixer::mixers().count(); ++i)
+         MixerToolBox::instance()->selectProfile((Mixer::mixers())[i])->increaseRefcount();
    clearMixerWidgets();
+
    if ( Mixer::mixers().count() > 0 ) {
       for (int i=0; i<Mixer::mixers().count(); ++i) {
          Mixer *mixer = (Mixer::mixers())[i];
+         // We've increased the refcount before clearing, so remember and decrease it again.
+         MixerToolBox::instance()->selectProfile(mixer)->decreaseRefcount();
          addMixerWidget(mixer->id());
       }
       bool dockingSucceded = updateDocking();
@@ -414,6 +457,39 @@ void KMixWindow::recreateGUI()
        updateDocking();  // -<- removes the DockIcon
        hide();
    }
+
+   if (current_tab >= 0) {
+      m_wsMixers->setCurrentIndex(current_tab);
+   }
+}
+
+
+/**
+* Create or recreate the Mixer GUI elements
+*/
+void KMixWindow::redrawMixer( const QString& mixer_ID )
+{
+    for ( int i=0; i<m_wsMixers->count() ; ++i )
+    {
+        QWidget *w = m_wsMixers->widget(i);
+        if ( w->inherits("KMixerWidget") )
+        {
+            KMixerWidget* kmw = (KMixerWidget*)w;
+            if ( kmw->mixer()->id() == mixer_ID )
+            {
+                kDebug(67100) << "KMixWindow::redrawMixer() " << mixer_ID << " is being redrawn";
+                kmw->loadConfig( KGlobal::config().data() );
+
+                // Is the below needed? It is done on startup so copied it here...
+                kmw->setTicks( m_showTicks );
+                kmw->setLabels( m_showLabels );
+
+                return;
+            }
+        }
+    }
+
+    kWarning(67100) << "KMixWindow::redrawMixer() Requested to redraw " << mixer_ID << " but I cannot find it :s";
 }
 
 
@@ -448,7 +524,7 @@ void KMixWindow::plugged( const char* driverName, const QString& /*udi*/, QStrin
     if ( mixer != 0 ) {
         kDebug(67100) << "Plugged: dev=" << dev << "\n";
         MixerToolBox::instance()->possiblyAddMixer(mixer);
-        recreateGUI();
+        recreateGUI(true);
     }
 
 // Test code for OSD. But OSD is postponed to KDE4.1
@@ -502,7 +578,7 @@ void KMixWindow::unplugged( const QString& udi)
                 text = i18n("The last soundcard was unplugged.");
                 KMixToolBox::notification("MasterFallback", text);
             }
-            recreateGUI();
+            recreateGUI(true);
             break;
         }
     }
@@ -544,7 +620,7 @@ void KMixWindow::addMixerWidget(const QString& mixer_ID)
    {
 //       kDebug(67100) << "KMixWindow::addMixerWidget() " << mixer_ID << " is being added";
       ViewBase::ViewFlags vflags = ViewBase::HasMenuBar;
-      if ( m_showMenubar ) {
+      if ( (_actionShowMenubar==0) || _actionShowMenubar->isChecked() ) {
             vflags |= ViewBase::MenuBarVisible;
       }
       if ( m_toplevelOrientation == Qt::Vertical ) {
@@ -639,31 +715,21 @@ void KMixWindow::slotDecreaseVolume()
 void KMixWindow::showVolumeDisplay()
 {
   Mixer* mixer = Mixer::getGlobalMasterMixer();
+  if ( mixer == 0 ) return; // e.g. when no soundcard is available
   MixDevice *md = Mixer::getGlobalMasterMD();
+  if ( md == 0 ) return; // shouldn't happen, but lets play safe
   int currentVolume = mixer->volume(md->id());
-  if (md->isMuted()) {
-    currentVolume = 0;
-  }
-  volumeDisplay->setValue(currentVolume);
-  volumeDisplay->show();
+  
+  osdWidget->setCurrentVolume(currentVolume, md->isMuted());
+  osdWidget->show();
+  osdWidget->activateOSD(); //Enable the hide timer
 
-  //FIXME, how to get this to work before it is displayed for the first time?
-  QDesktopWidget* desktop = KApplication::kApplication()->desktop();
-  //Getting QRect of the screen where cursor is positioned
-  QRect rect = desktop->screenGeometry(QCursor::pos());
-  int width = (rect.width()/2) - (volumeDisplay->width()/2);
-  int height = (rect.height()/2) - (volumeDisplay->height()/2);
-  width += rect.x();
-  height += rect.y();
-  volumeDisplay->move(width, height);
-
-  volumeDisplayTimer->setInterval(2000);
-  volumeDisplayTimer->start();
-}
-
-void KMixWindow::slotHideVolumeDisplay()
-{
-  volumeDisplay->hide();
+  //Center the OSD
+  QRect rect = KApplication::kApplication()->desktop()->screenGeometry(QCursor::pos());
+  QSize size = osdWidget->sizeHint();
+  int posX = rect.x() + (rect.width() - size.width()) / 2;
+  int posY = rect.y() + 4 * rect.height() / 5;
+  osdWidget->setGeometry(posX, posY, size.width(), size.height());
 }
 
 void KMixWindow::slotMute()
@@ -672,7 +738,7 @@ void KMixWindow::slotMute()
   if ( mixer == 0 ) return; // e.g. when no soundcard is available
   MixDevice *md = Mixer::getGlobalMasterMD();
   if ( md == 0 ) return; // shouldn't happen, but lets play safe
-  mixer->toggleMute(md->id());
+  mixer->toggleMute(md->id()); 
   showVolumeDisplay();
 }
 
@@ -741,7 +807,7 @@ void KMixWindow::applyPrefs( KMixPrefDlg *prefDlg )
    }
 
    if ( labelsHasChanged || ticksHasChanged || dockwidgetHasChanged || toplevelOrientationHasChanged || systrayPopupHasChanged) {
-      recreateGUI();
+      recreateGUI(true);
    }
 
    this->repaint(); // make KMix look fast (saveConfig() often uses several seconds)
@@ -789,6 +855,12 @@ void KMixWindow::slotConfigureCurrentView()
     if (view) view->configureView();
 }
 
+void KMixWindow::slotSelectMaster()
+{
+   DialogSelectMaster* dsm = new DialogSelectMaster(Mixer::getGlobalMasterMixer());
+   if (dsm) dsm->show();
+}
+
 void KMixWindow::newMixerShown(int /*tabIndex*/ ) {
    KMixerWidget* mw = (KMixerWidget*)m_wsMixers->currentWidget();
    if (mw) {
