@@ -81,6 +81,8 @@ protected:
   virtual void setRecsrcHW( const QString& id, bool on) = 0;
   //virtual bool isRecsrcHW( const QString& id ) = 0;
 
+  virtual bool moveStream( const QString& id, const QString& destId );
+
   /// Overwrite in the backend if the backend can see changes without polling
   virtual bool needsPolling() { return true; }
 
@@ -131,6 +133,10 @@ protected:
 
 signals:
   void controlChanged( void );
+  void controlsReconfigured( const QString& mixer_ID );
+
+public slots:
+  virtual void reinit() {};
 
 protected slots:
   virtual void readSetFromHW();
