@@ -17,27 +17,29 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this program; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #ifndef MIXER_SUN_H
 #define MIXER_SUN_H
 
-#include <qstring.h>
+#include <QString>
 
 #include "mixer_backend.h"
 
 class Mixer_SUN : public Mixer_Backend
 {
 public:
-  Mixer_SUN(int devnum);
+  Mixer_SUN(Mixer *mixer, int devnum);
   virtual ~Mixer_SUN();
 
   virtual QString errorText(int mixer_error);
-  virtual int readVolumeFromHW( int devnum, Volume& volume );
-  virtual int writeVolumeToHW( int devnum, Volume& volume );
-  bool setRecsrcHW( int devnum, bool on );
-  bool isRecsrcHW( int devnum );
+  virtual int readVolumeFromHW( const QString& id, MixDevice *md  );
+  virtual int writeVolumeToHW ( const QString& id, MixDevice *md  );
+  void setRecsrcHW              ( const QString& id, bool on );
+  bool isRecsrcHW               ( const QString& id );
+
+  virtual QString getDriverName();
 
 protected:
   virtual int open();
