@@ -34,6 +34,7 @@
 #include <QWidget>
 #include <QVBoxLayout>
 #include <QHBoxLayout>
+//#include <QFormLayout>
 
 
 
@@ -50,24 +51,25 @@ ViewSliders::ViewSliders(QWidget* parent, const char* name, Mixer* mixer, ViewBa
       _layoutSliders = new QVBoxLayout();
       _layoutSliders->setAlignment(Qt::AlignVCenter|Qt::AlignLeft);
       _layoutMDW->addItem( _layoutSliders );
-      _layoutEnum = new QVBoxLayout(); // always vertical!
-      _layoutMDW->addItem( _layoutEnum );
    }
-   else {
+   else
+   {
       _layoutMDW = new QHBoxLayout(this);
       _layoutMDW->setAlignment(Qt::AlignLeft|Qt::AlignTop);
       _layoutSliders = new QHBoxLayout();
       _layoutSliders->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
       _layoutMDW->addItem( _layoutSliders );
       // Place enums in an own box right from the sliders.
-      _layoutEnum = new QVBoxLayout();
-      _layoutMDW->addItem( _layoutEnum );
    }
+  _layoutEnum = new QVBoxLayout(); // new QFormLayout();
+  _layoutMDW->addLayout( _layoutEnum );
+
    _layoutMDW->setSpacing(0);
     setMixSet();
 }
 
-ViewSliders::~ViewSliders() {
+ViewSliders::~ViewSliders()
+{
   qDeleteAll(_separators);
 }
 
@@ -87,7 +89,7 @@ QWidget* ViewSliders::add(MixDevice *md)
                this,         // parent
                this          // View widget
       );
-      _layoutEnum->addWidget(mdw);
+	  _layoutEnum->addWidget(mdw);
    } // an enum
    else {
       // add a separator before the device
@@ -119,10 +121,20 @@ QWidget* ViewSliders::add(MixDevice *md)
 }
 
 
-void ViewSliders::setMixSet()
+void ViewSliders::_setMixSet()
 {
-    const MixSet& mixset = _mixer->getMixSet();
+   const MixSet& mixset = _mixer->getMixSet();
 
+   if ( _mixer->dynamic() ) {
+      // We will be recreating our sliders, so make sure we trash all the separators too.
+      qDeleteAll(_separators);
+      _separators.clear();
+      // Our _layoutSliders now should only contain spacer widgets from the addSpacing() calls in add() above.
+      // We need to trash those too otherwise all sliders gradually migrate away from the edge :p
+      QLayoutItem *li;
+      while ( ( li = _layoutSliders->takeAt(0) ) )
+         delete li;
+   }
 
    // This method iterates the controls from the Profile
    // Each control is checked, whether it is also contained in the mixset, and
@@ -135,7 +147,7 @@ void ViewSliders::setMixSet()
       if ( control->tab == id() ) {
          // The TabName of the control matches this View name (!! attention: Better use some ID, due to i18n() )
          bool isUsed = false;
-   
+
          QRegExp idRegexp(control->id);
          //kDebug(67100) << "ViewSliders::setMixSet(): Check GUIProfile id==" << control->id << "\n";
          // The following for-loop could be simplified by using a std::find_if
@@ -184,15 +196,18 @@ void ViewSliders::constructionFinished() {
     configurationUpdate();
 }
 
+
 void ViewSliders::configurationUpdate() {
    // Adjust height of top part by setting it to the maximum of all mdw's
-   int topPartExtent = 0;
-   int bottomPartExtent = 0;
+   bool haveCaptureLEDs = false;
+   int labelExtent = 0;
+   bool haveMuteButtons = false;
    for ( int i=0; i<_mdws.count(); i++ ) {
       MDWSlider* mdw = ::qobject_cast<MDWSlider*>(_mdws[i]);
       if ( mdw && mdw->isVisibleTo(this) ) {
-         if ( mdw->playbackExtentHint() > topPartExtent ) topPartExtent = mdw->playbackExtentHint();
-         if ( mdw->captureExtentHint() > bottomPartExtent ) bottomPartExtent = mdw->captureExtentHint();
+		 if ( mdw->labelExtentHint() > labelExtent ) labelExtent = mdw->labelExtentHint();
+		 haveCaptureLEDs = haveCaptureLEDs || mdw->hasCaptureLED();
+		 haveMuteButtons = haveMuteButtons || mdw->hasMuteButton();
       }
    }
    //kDebug(67100) << "topPartExtent is " << topPartExtent;
@@ -200,8 +215,9 @@ void ViewSliders::configurationUpdate() {
    for ( int i=0; i<_mdws.count(); i++ ) {
       MDWSlider* mdw = ::qobject_cast<MDWSlider*>(_mdws[i]);
       if ( mdw ) {
-         mdw->setPlaybackExtent(topPartExtent);
-         mdw->setCaptureExtent(bottomPartExtent);
+		 mdw->setLabelExtent(labelExtent);
+		 mdw->setMuteButtonSpace(haveMuteButtons);
+		 mdw->setCaptureLEDSpace(haveCaptureLEDs);
          bool thisControlIsVisible = mdw->isVisibleTo(this);
          bool showSeparator = ( firstVisibleControlFound && thisControlIsVisible);
          if ( _separators.contains( mdw->mixDevice()->id() )) {
